@@ -3,7 +3,7 @@
 
  * Copyright (C) 1997, 1998, 1999, 2002, 2003  Seth A. Robinson
  * Copyright (C) 2005, 2006  Dan Walma
- * Copyright (C) 2005, 2007  Sylvain Beucler
+ * Copyright (C) 2005, 2007, 2008, 2009  Sylvain Beucler
 
  * This file is part of GNU FreeDink
 
@@ -22,6 +22,10 @@
  * <http://www.gnu.org/licenses/>.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -33,6 +37,7 @@
 
 #include "dinkvar.h"
 #include "game_engine.h"
+#include "screen.h"
 #include "freedink.h"
 #include "gfx.h"
 /* For printing strings in debug mode */
@@ -43,6 +48,7 @@
 #include "log.h"
 #include "rect.h"
 #include "dinkc_console.h"
+#include "input.h"
 
 #include "update_frame.h"
 
@@ -83,12 +89,8 @@ void updateFrame( void )
        copying the screen to lpDDSTrick2 */
     /*bool*/int get_frame = /*false*/0;
 
-	int h,max_s,j;
-	//HBRUSH brush;
-	/*BOOL*/int bs[MAX_SPRITES_AT_ONCE];
-	
+	int max_s;
 	int rank[MAX_SPRITES_AT_ONCE];
-	int highest_sprite;
 	
 	abort_this_flip = /*false*/0;
 	
@@ -109,28 +111,27 @@ trigger_start:
 	}
 	
 	if (GetKeyboard('d') && (GetKeyboard(SDLK_LALT) || GetKeyboard(SDLK_RALT)))
-	{	
-		if (debug_mode)
-		{
-			debug_mode = /*false*/0;
-		}
-		else 
-		{
-			strcpy(last_debug, "");
-			debug_mode = /*true*/1;	  	  
-			
-		}
-		
+	  {	
+	    if (debug_mode)
+	      {
+		log_debug_off();
+	      }
+	    else
+	      {
+		log_debug_on();
+	      }
 	}
 		  
 	
 	
 	if (GetKeyboard('q') && (GetKeyboard(SDLK_LALT) || GetKeyboard(SDLK_RALT)))
 	{
-		//shutdown game
-	//	PostMessage(hWndMain, WM_CLOSE, 0, 0);
-		finiObjects();
-	    return;
+	  //shutdown game
+	  //	PostMessage(hWndMain, WM_CLOSE, 0, 0);
+	  SDL_Event ev;
+	  ev.type = SDL_QUIT;
+	  SDL_PushEvent(&ev);
+	  return;
 	}
 	
 	if (mode == 1) Scrawl_OnMouseInput(); else
@@ -280,7 +281,7 @@ trigger_start:
 	//figure out frame rate
 	drawthistime = /*true*/1;
 	
-	if (item_screen)
+	if (show_inventory)
 	{
 		process_item();
 		return;
@@ -301,89 +302,18 @@ trigger_start:
 	
 	
 	if (plane_process)
-	{
-		
-		memset(&bs,0,sizeof(bs));
-		
-		max_s = last_sprite_created;
-		
-		int height;
-		int r1;
-		for (r1 = 1; r1 < max_s+1; r1++)
-		{
-		  int h1;
-			highest_sprite = 22024; //more than it could ever be
-			
-			rank[r1] = 0;
-			
-			for (h1 = 1; h1 < max_s+1; h1++)
-			{
-				if (spr[h1].active) if (spr[h1].disabled == /*false*/0)
-				{ 
-					if (bs[h1] == /*FALSE*/0)
-					{
-						//Msg( "Ok,  %d is %d", h1,(spr[h1].y + k[spr[h1].pic].yoffset) );
-						if (spr[h1].que != 0) height = spr[h1].que; else height = spr[h1].y;
-						if ( height < highest_sprite )
-						{
-							highest_sprite = height;
-							rank[r1] = h1;
-						}
-						
-					}
-					
-				}
-				
-			}
-			if (rank[r1] != 0)	
-				bs[rank[r1]] = /*TRUE*/1;
-			
-		}
-		
-	} else
-	{
-		//not processing planes
-		max_s = MAX_SPRITES_AT_ONCE;
-	}
+	  {
+	    max_s = last_sprite_created;
+	    screen_rank_game_sprites(rank);
+	  }
+	else
+	  {
+	    //not processing planes
+	    max_s = MAX_SPRITES_AT_ONCE;
+	  }
 	
-/*     rcRect.left = 0; */
-/*     rcRect.top = 0; */
-/*     rcRect.right = x; */
-/*     rcRect.bottom = y; */
-	
-	{
-		
-		
-		//Blit from Two, which holds the base scene.
-/* 		while( 1 ) */
-/* 		{ */
-/* 			ddrval = lpDDSBack->BltFast( 0, 0, lpDDSTwo, */
-/* 				&rcRect, DDBLTFAST_NOCOLORKEY); */
-			// GFX
-			SDL_BlitSurface(GFX_lpDDSTwo, NULL, GFX_lpDDSBack, NULL);
-			
-/* 			if( ddrval == DD_OK ) */
-/* 			{ */
-/* 				break; */
-/* 			} */
-/* 			if( ddrval == DDERR_SURFACELOST ) */
-/* 			{ */
-/* 				ddrval = restoreAll(); */
-/* 				if( ddrval != DD_OK ) */
-/* 				{ */
-/* 					//    return; */
-/* 				} */
-				
-/* 				goto demon; */
-/* 			} */
-/* 			if( ddrval != DDERR_WASSTILLDRAWING ) */
-/* 			{ */
-/* 				dderror(ddrval); */
-/* 				return; */
-/* 			} */
-/* 		} */
-		
-	}
+	//Blit from Two, which holds the base scene.
+	SDL_BlitSurface(GFX_lpDDSTwo, NULL, GFX_lpDDSBack, NULL);
 	
 	
 	if (stop_entire_game == 1)
@@ -405,16 +335,19 @@ trigger_start:
 	
 	
 	
-	
-	for ( j = 1; j < max_s+1; j++)
+	int j = 0;
+	for (; j <= max_s; j++)
 	{
 		//h  = 1;
+		int h = 0;
 		if (plane_process)
-			h = rank[j]; else h = j;
+		  h = rank[j];
+		else
+		  h = j;
 		//Msg( "Ok, rank %d is %d", j,h);
 		
 		if (h > 0) 
-			if (spr[h].active)
+			if (spr[h].active && spr[h].disabled == 0)
 			{
 				
 				//check_sprite_status_full(h);
@@ -511,7 +444,7 @@ animate:
 				
 				if (flub_mode != -500)
 				{
-					Msg("move result is %d", flub_mode);
+					log_debug("move result is %d", flub_mode);
 					move_result = flub_mode;
 					flub_mode = -500;
 				}
@@ -755,9 +688,9 @@ past:
 						// GFX
 						SDL_FillRect(GFX_lpDDSBack, &GFX_box_crap,
 							     SDL_MapRGB(GFX_lpDDSBack->format,
-									cur_screen_palette[100].r,
-									cur_screen_palette[100].g,
-									cur_screen_palette[100].b));
+									GFX_real_pal[100].r,
+									GFX_real_pal[100].g,
+									GFX_real_pal[100].b));
 
 						
 						for (oo=0; oo <  spr[h].moveman+1; oo++)
@@ -778,9 +711,9 @@ past:
 							// GFX
 							SDL_FillRect(GFX_lpDDSBack, &GFX_box_crap,
 								     SDL_MapRGB(GFX_lpDDSBack->format,
-										cur_screen_palette[50].r,
-										cur_screen_palette[50].g,
-										cur_screen_palette[50].b));
+										GFX_real_pal[50].r,
+										GFX_real_pal[50].g,
+										GFX_real_pal[50].b));
 							
 						}
 /* 						ddbltfx.dwFillColor = 1; */
@@ -799,13 +732,13 @@ past:
 						// GFX
 						SDL_FillRect(GFX_lpDDSBack, &GFX_box_crap,
 							     SDL_MapRGB(GFX_lpDDSBack->format,
-									cur_screen_palette[1].r,
-									cur_screen_palette[1].g,
-									cur_screen_palette[1].b));
+									GFX_real_pal[1].r,
+									GFX_real_pal[1].g,
+									GFX_real_pal[1].b));
 					}
 				}         
 }
- }								
+} /* for 0->max_s */
 
  
 	if (mode == 0)
@@ -850,7 +783,7 @@ past:
 		int scr = load_script("START",1000, /*true*/1);
 		if (locate(scr, "MAIN") == /*false*/0)
 		{
-			Msg("Error: Can't locate MAIN in script START!");
+			log_error("Can't locate MAIN in script START!");
 		}
 		run_script(scr);
 		mode = 1;
@@ -958,7 +891,7 @@ past:
 		//let's add the key info to it.
 		for (x = 0; x < 256; x++)
 		  if (GetKeyboard(x))
-		    sprintf(msg, "%s (Key %i)",msg,x);
+		    sprintf(msg + strlen(msg), " (Key %i)", x);
 	      }
 	    if (mode == 3)
 	      {
@@ -991,28 +924,17 @@ past:
 	  print_text_wrap_debug(retval, 20, 360);
 	}
 
-		
-		for ( j = 1; j < max_s+1; j++)
-		{
-			if (plane_process)
-				h = rank[j]; else h = j;
-			if (h > 0) 
-				if (spr[h].active)
-				{
-					if (spr[h].brain == 8) 
-					{
-						//Msg("Drawing text %d..", h);
-/*  						text_draw(h,hdc); */
- 						text_draw(h);
-					}
-
-                }
-/*         } */
-        
-        
-/*         lpDDSBack->ReleaseDC(hdc); */
-        
-    }
+	int j2 = 0;
+	for (; j2 <= max_s; j2++)
+	  {
+	    int h = 0;
+	    if (plane_process)
+	      h = rank[j2];
+	    else
+	      h = j2;
+	    if (h > 0 && spr[h].active && spr[h].brain == 8)
+	      text_draw(h);
+	  }
     
     
 	if (talk.active) process_talk();
